@@ -37,11 +37,11 @@ function get_category_link( $category ) {
  * @param string $separator Optional, default is '/'. How to separate categories.
  * @param bool $nicename Optional, default is false. Whether to use nice name for display.
  * @param array $visited Optional. Already linked to categories to prevent duplicates.
- * @return string
+ * @return string|WP_Error A list of category parents on success, WP_Error on failure.
  */
 function get_category_parents( $id, $link = false, $separator = '/', $nicename = false, $visited = array() ) {
 	$chain = '';
-	$parent = &get_category( $id );
+	$parent = get_term( $id, 'category' );
 	if ( is_wp_error( $parent ) )
 		return $parent;
 
@@ -56,7 +56,7 @@ function get_category_parents( $id, $link = false, $separator = '/', $nicename =
 	}
 
 	if ( $link )
-		$chain .= '<a href="' . get_category_link( $parent->term_id ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $parent->name ) ) . '">'.$name.'</a>' . $separator;
+		$chain .= '<a href="' . esc_url( get_category_link( $parent->term_id ) ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $parent->name ) ) . '">'.$name.'</a>' . $separator;
 	else
 		$chain .= $name.$separator;
 	return $chain;
@@ -73,7 +73,7 @@ function get_category_parents( $id, $link = false, $separator = '/', $nicename =
  */
 function get_the_category( $id = false ) {
 	$categories = get_the_terms( $id, 'category' );
-	if ( ! $categories )
+	if ( ! $categories || is_wp_error( $categories ) )
 		$categories = array();
 
 	$categories = array_values( $categories );
@@ -82,7 +82,7 @@ function get_the_category( $id = false ) {
 		_make_cat_compat( $categories[$key] );
 	}
 
-	// Filter name is plural because we return alot of categories not just one
+	// Filter name is plural because we return alot of categories (possibly more than #13237) not just one
 	return apply_filters( 'get_the_categories', $categories );
 }
 
@@ -131,11 +131,11 @@ function _usort_terms_by_ID( $a, $b ) {
  * @since 0.71
  *
  * @param int $cat_ID Category ID.
- * @return string Category name.
+ * @return string|WP_Error Category name on success, WP_Error on failure.
  */
 function get_the_category_by_ID( $cat_ID ) {
 	$cat_ID = (int) $cat_ID;
-	$category = &get_category( $cat_ID );
+	$category = get_term( $cat_ID, 'category' );
 	if ( is_wp_error( $category ) )
 		return $category;
 	return $category->name;
@@ -153,10 +153,10 @@ function get_the_category_by_ID( $cat_ID ) {
  */
 function get_the_category_list( $separator = '', $parents='', $post_id = false ) {
 	global $wp_rewrite;
-	$categories = get_the_category( $post_id );
-	if ( !is_object_in_taxonomy( get_post_type( $post_id ), 'category' ) )
+	if ( ! is_object_in_taxonomy( get_post_type( $post_id ), 'category' ) )
 		return apply_filters( 'the_category', '', $separator, $parents );
 
+	$categories = get_the_category( $post_id );
 	if ( empty( $categories ) )
 		return apply_filters( 'the_category', __( 'Uncategorized' ), $separator, $parents );
 
@@ -171,17 +171,17 @@ function get_the_category_list( $separator = '', $parents='', $post_id = false )
 				case 'multiple':
 					if ( $category->parent )
 						$thelist .= get_category_parents( $category->parent, true, $separator );
-					$thelist .= '<a href="' . get_category_link( $category->term_id ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a></li>';
+					$thelist .= '<a href="' . esc_url( get_category_link( $category->term_id ) ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a></li>';
 					break;
 				case 'single':
-					$thelist .= '<a href="' . get_category_link( $category->term_id ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>';
+					$thelist .= '<a href="' . esc_url( get_category_link( $category->term_id ) ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>';
 					if ( $category->parent )
 						$thelist .= get_category_parents( $category->parent, false, $separator );
 					$thelist .= $category->name.'</a></li>';
 					break;
 				case '':
 				default:
-					$thelist .= '<a href="' . get_category_link( $category->term_id ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a></li>';
+					$thelist .= '<a href="' . esc_url( get_category_link( $category->term_id ) ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a></li>';
 			}
 		}
 		$thelist .= '</ul>';
@@ -194,17 +194,17 @@ function get_the_category_list( $separator = '', $parents='', $post_id = false )
 				case 'multiple':
 					if ( $category->parent )
 						$thelist .= get_category_parents( $category->parent, true, $separator );
-					$thelist .= '<a href="' . get_category_link( $category->term_id ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a>';
+					$thelist .= '<a href="' . esc_url( get_category_link( $category->term_id ) ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a>';
 					break;
 				case 'single':
-					$thelist .= '<a href="' . get_category_link( $category->term_id ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>';
+					$thelist .= '<a href="' . esc_url( get_category_link( $category->term_id ) ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>';
 					if ( $category->parent )
 						$thelist .= get_category_parents( $category->parent, false, $separator );
 					$thelist .= "$category->name</a>";
 					break;
 				case '':
 				default:
-					$thelist .= '<a href="' . get_category_link( $category->term_id ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a>';
+					$thelist .= '<a href="' . esc_url( get_category_link( $category->term_id ) ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), $category->name ) ) . '" ' . $rel . '>' . $category->name.'</a>';
 			}
 			++$i;
 		}
@@ -212,7 +212,6 @@ function get_the_category_list( $separator = '', $parents='', $post_id = false )
 	return apply_filters( 'the_category', $thelist, $separator, $parents );
 }
 
-
 /**
  * Check if the current post in within any of the given categories.
  *
@@ -226,16 +225,17 @@ function get_the_category_list( $separator = '', $parents='', $post_id = false )
  * As of 2.7, the function can be used anywhere if it is provided a post ID or post object.
  *
  * @since 1.2.0
+ * @uses has_category()
  *
  * @param int|string|array $category Category ID, name or slug, or array of said.
- * @param int|object $_post Optional. Post to check instead of the current post. (since 2.7.0)
+ * @param int|object $post Optional. Post to check instead of the current post. (since 2.7.0)
  * @return bool True if the current post is in any of the given categories.
  */
 function in_category( $category, $post = null ) {
 	if ( empty( $category ) )
 		return false;
 
-	return has_term( $category, 'category', $post );
+	return has_category( $category, $post );
 }
 
 /**
@@ -272,7 +272,6 @@ function category_description( $category = 0 ) {
  *     'orderby' (string) default is 'ID' - What column to use for ordering the
  * categories.
  *     'order' (string) default is 'ASC' - What direction to order categories.
- *     'show_last_update' (bool|int) default is 0 - See {@link get_categories()}
  *     'show_count' (bool|int) default is 0 - Whether to show how many posts are
  * in the category.
  *     'hide_empty' (bool|int) default is 1 - Whether to hide categories that
@@ -302,7 +301,7 @@ function wp_dropdown_categories( $args = '' ) {
 	$defaults = array(
 		'show_option_all' => '', 'show_option_none' => '',
 		'orderby' => 'id', 'order' => 'ASC',
-		'show_last_update' => 0, 'show_count' => 0,
+		'show_count' => 0,
 		'hide_empty' => 1, 'child_of' => 0,
 		'exclude' => '', 'echo' => 1,
 		'selected' => 0, 'hierarchical' => 0,
@@ -326,7 +325,6 @@ function wp_dropdown_categories( $args = '' ) {
 		$r['pad_counts'] = true;
 	}
 
-	$r['include_last_update_time'] = $r['show_last_update'];
 	extract( $r );
 
 	$tab_index_attribute = '';
@@ -369,10 +367,10 @@ function wp_dropdown_categories( $args = '' ) {
 
 		$output .= walk_category_dropdown_tree( $categories, $depth, $r );
 	}
+
 	if ( ! $r['hide_if_empty'] || ! empty($categories) )
 		$output .= "</select>\n";
 
-
 	$output = apply_filters( 'wp_dropdown_cats', $output );
 
 	if ( $echo )
@@ -389,8 +387,6 @@ function wp_dropdown_categories( $args = '' ) {
  *     'orderby' (string) default is 'ID' - What column to use for ordering the
  * categories.
  *     'order' (string) default is 'ASC' - What direction to order categories.
- *     'show_last_update' (bool|int) default is 0 - See {@link
- * walk_category_dropdown_tree()}
  *     'show_count' (bool|int) default is 0 - Whether to show how many posts are
  * in the category.
  *     'hide_empty' (bool|int) default is 1 - Whether to hide categories that
@@ -418,7 +414,7 @@ function wp_list_categories( $args = '' ) {
 	$defaults = array(
 		'show_option_all' => '', 'show_option_none' => __('No categories'),
 		'orderby' => 'name', 'order' => 'ASC',
-		'show_last_update' => 0, 'style' => 'list',
+		'style' => 'list',
 		'show_count' => 0, 'hide_empty' => 1,
 		'use_desc_for_title' => 1, 'child_of' => 0,
 		'feed' => '', 'feed_type' => '',
@@ -434,9 +430,6 @@ function wp_list_categories( $args = '' ) {
 	if ( !isset( $r['pad_counts'] ) && $r['show_count'] && $r['hierarchical'] )
 		$r['pad_counts'] = true;
 
-	if ( isset( $r['show_date'] ) )
-		$r['include_last_update_time'] = $r['show_date'];
-
 	if ( true == $r['hierarchical'] ) {
 		$r['exclude_tree'] = $r['exclude'];
 		$r['exclude'] = '';
@@ -464,15 +457,18 @@ function wp_list_categories( $args = '' ) {
 				$output .= $show_option_none;
 		}
 	} else {
-		if( !empty( $show_option_all ) )
+		if ( ! empty( $show_option_all ) ) {
+			$posts_page = ( 'page' == get_option( 'show_on_front' ) && get_option( 'page_for_posts' ) ) ? get_permalink( get_option( 'page_for_posts' ) ) : home_url( '/' );
+			$posts_page = esc_url( $posts_page );
 			if ( 'list' == $style )
-				$output .= '<li><a href="' .  get_bloginfo( 'url' )  . '">' . $show_option_all . '</a></li>';
+				$output .= "<li><a href='$posts_page'>$show_option_all</a></li>";
 			else
-				$output .= '<a href="' .  get_bloginfo( 'url' )  . '">' . $show_option_all . '</a>';
+				$output .= "<a href='$posts_page'>$show_option_all</a>";
+		}
 
 		if ( empty( $r['current_category'] ) && ( is_category() || is_tax() || is_tag() ) ) {
 			$current_term_object = get_queried_object();
-			if ( $r['taxonomy'] == $current_term_object->taxonomy )
+			if ( $current_term_object && $r['taxonomy'] === $current_term_object->taxonomy )
 				$r['current_category'] = get_queried_object_id();
 		}
 
@@ -533,7 +529,7 @@ function wp_tag_cloud( $args = '' ) {
 
 	$tags = get_terms( $args['taxonomy'], array_merge( $args, array( 'orderby' => 'count', 'order' => 'DESC' ) ) ); // Always query top tags
 
-	if ( empty( $tags ) )
+	if ( empty( $tags ) || is_wp_error( $tags ) )
 		return;
 
 	foreach ( $tags as $key => $tag ) {
@@ -578,7 +574,6 @@ function default_topic_count_scale( $count ) {
 	return round(log10($count + 1) * 100);
 }
 
-
 /**
  * Generates a tag cloud (heatmap) from provided data.
  *
@@ -611,7 +606,6 @@ function default_topic_count_scale( $count ) {
  * @return string
  */
 function wp_generate_tag_cloud( $tags, $args = '' ) {
-	global $wp_rewrite;
 	$defaults = array(
 		'smallest' => 8, 'largest' => 22, 'unit' => 'pt', 'number' => 0,
 		'format' => 'flat', 'separator' => "\n", 'orderby' => 'name', 'order' => 'ASC',
@@ -642,9 +636,9 @@ function wp_generate_tag_cloud( $tags, $args = '' ) {
 		} else {
 			// SQL cannot save you; this is a second (potentially different) sort on a subset of data.
 			if ( 'name' == $orderby )
-				uasort( $tags, create_function('$a, $b', 'return strnatcasecmp($a->name, $b->name);') );
+				uasort( $tags, '_wp_object_name_sort_cb' );
 			else
-				uasort( $tags, create_function('$a, $b', 'return ($a->count > $b->count);') );
+				uasort( $tags, '_wp_object_count_sort_cb' );
 
 			if ( 'DESC' == $order )
 				$tags = array_reverse( $tags, true );
@@ -678,8 +672,8 @@ function wp_generate_tag_cloud( $tags, $args = '' ) {
 		$tag_link = '#' != $tag->link ? esc_url( $tag->link ) : '#';
 		$tag_id = isset($tags[ $key ]->id) ? $tags[ $key ]->id : $key;
 		$tag_name = $tags[ $key ]->name;
-		$a[] = "<a href='$tag_link' class='tag-link-$tag_id' title='" . esc_attr( call_user_func( $topic_count_text_callback, $real_count ) ) . "' style='font-size: " .
-			( $smallest + ( ( $count - $min_count ) * $font_step ) )
+		$a[] = "<a href='$tag_link' class='tag-link-$tag_id' title='" . esc_attr( call_user_func( $topic_count_text_callback, $real_count, $tag, $args ) ) . "' style='font-size: " .
+			str_replace( ',', '.', ( $smallest + ( ( $count - $min_count ) * $font_step ) ) )
 			. "$unit;'>$tag_name</a>";
 	}
 
@@ -704,22 +698,22 @@ function wp_generate_tag_cloud( $tags, $args = '' ) {
 }
 
 /**
- * Callback for comparing tags based on name
+ * Callback for comparing objects based on name
  *
  * @since 3.1.0
  * @access private
  */
-function _wp_tag_cloud_name_sort_cb( $a, $b ) {
+function _wp_object_name_sort_cb( $a, $b ) {
 	return strnatcasecmp( $a->name, $b->name );
 }
 
 /**
- * Callback for comparing tags based on count
+ * Callback for comparing objects based on count
  *
  * @since 3.1.0
  * @access private
  */
-function _wp_tag_cloud_count_sort_cb( $a, $b ) {
+function _wp_object_count_sort_cb( $a, $b ) {
 	return ( $a->count > $b->count );
 }
 
@@ -772,6 +766,8 @@ function walk_category_dropdown_tree() {
  */
 class Walker_Category extends Walker {
 	/**
+	 * What the class handles.
+	 *
 	 * @see Walker::$tree_type
 	 * @since 2.1.0
 	 * @var string
@@ -779,6 +775,8 @@ class Walker_Category extends Walker {
 	var $tree_type = 'category';
 
 	/**
+	 * Database fields to use.
+	 *
 	 * @see Walker::$db_fields
 	 * @since 2.1.0
 	 * @todo Decouple this
@@ -787,14 +785,18 @@ class Walker_Category extends Walker {
 	var $db_fields = array ('parent' => 'parent', 'id' => 'term_id');
 
 	/**
+	 * Starts the list before the elements are added.
+	 *
 	 * @see Walker::start_lvl()
+	 *
 	 * @since 2.1.0
 	 *
 	 * @param string $output Passed by reference. Used to append additional content.
-	 * @param int $depth Depth of category. Used for tab indentation.
-	 * @param array $args Will only append content if style argument value is 'list'.
+	 * @param int    $depth  Depth of category. Used for tab indentation.
+	 * @param array  $args   An array of arguments. Will only append content if style argument value is 'list'.
+	 *                       @see wp_list_categories()
 	 */
-	function start_lvl(&$output, $depth, $args) {
+	function start_lvl( &$output, $depth = 0, $args = array() ) {
 		if ( 'list' != $args['style'] )
 			return;
 
@@ -803,14 +805,18 @@ class Walker_Category extends Walker {
 	}
 
 	/**
+	 * Ends the list of after the elements are added.
+	 *
 	 * @see Walker::end_lvl()
+	 *
 	 * @since 2.1.0
 	 *
 	 * @param string $output Passed by reference. Used to append additional content.
-	 * @param int $depth Depth of category. Used for tab indentation.
-	 * @param array $args Will only append content if style argument value is 'list'.
+	 * @param int    $depth  Depth of category. Used for tab indentation.
+	 * @param array  $args   An array of arguments. Will only append content if style argument value is 'list'.
+	 *                       @wsee wp_list_categories()
 	 */
-	function end_lvl(&$output, $depth, $args) {
+	function end_lvl( &$output, $depth = 0, $args = array() ) {
 		if ( 'list' != $args['style'] )
 			return;
 
@@ -819,20 +825,24 @@ class Walker_Category extends Walker {
 	}
 
 	/**
+	 * Start the element output.
+	 *
 	 * @see Walker::start_el()
+	 *
 	 * @since 2.1.0
 	 *
-	 * @param string $output Passed by reference. Used to append additional content.
+	 * @param string $output   Passed by reference. Used to append additional content.
 	 * @param object $category Category data object.
-	 * @param int $depth Depth of category in reference to parents.
-	 * @param array $args
+	 * @param int    $depth    Depth of category in reference to parents. Default 0.
+	 * @param array  $args     An array of arguments. @see wp_list_categories()
+	 * @param int    $id       ID of the current category.
 	 */
-	function start_el(&$output, $category, $depth, $args) {
+	function start_el( &$output, $category, $depth = 0, $args = array(), $id = 0 ) {
 		extract($args);
 
 		$cat_name = esc_attr( $category->name );
 		$cat_name = apply_filters( 'list_cats', $cat_name, $category );
-		$link = '<a href="' . esc_attr( get_term_link($category) ) . '" ';
+		$link = '<a href="' . esc_url( get_term_link($category) ) . '" ';
 		if ( $use_desc_for_title == 0 || empty($category->description) )
 			$link .= 'title="' . esc_attr( sprintf(__( 'View all posts filed under %s' ), $cat_name) ) . '"';
 		else
@@ -846,7 +856,7 @@ class Walker_Category extends Walker {
 			if ( empty($feed_image) )
 				$link .= '(';
 
-			$link .= '<a href="' . get_term_feed_link( $category->term_id, $category->taxonomy, $feed_type ) . '"';
+			$link .= '<a href="' . esc_url( get_term_feed_link( $category->term_id, $category->taxonomy, $feed_type ) ) . '"';
 
 			if ( empty($feed) ) {
 				$alt = ' alt="' . sprintf(__( 'Feed for all posts filed under %s' ), $cat_name ) . '"';
@@ -873,9 +883,6 @@ class Walker_Category extends Walker {
 		if ( !empty($show_count) )
 			$link .= ' (' . intval($category->count) . ')';
 
-		if ( !empty($show_date) )
-			$link .= ' ' . gmdate('Y-m-d', $category->last_update_timestamp);
-
 		if ( 'list' == $args['style'] ) {
 			$output .= "\t<li";
 			$class = 'cat-item cat-item-' . $category->term_id;
@@ -894,15 +901,18 @@ class Walker_Category extends Walker {
 	}
 
 	/**
+	 * Ends the element output, if needed.
+	 *
 	 * @see Walker::end_el()
+	 *
 	 * @since 2.1.0
 	 *
 	 * @param string $output Passed by reference. Used to append additional content.
-	 * @param object $page Not used.
-	 * @param int $depth Depth of category. Not used.
-	 * @param array $args Only uses 'list' for whether should append to output.
+	 * @param object $page   Not used.
+	 * @param int    $depth  Depth of category. Not used.
+	 * @param array  $args   An array of arguments. Only uses 'list' for whether should append to output. @see wp_list_categories()
 	 */
-	function end_el(&$output, $page, $depth, $args) {
+	function end_el( &$output, $page, $depth = 0, $args = array() ) {
 		if ( 'list' != $args['style'] )
 			return;
 
@@ -935,15 +945,17 @@ class Walker_CategoryDropdown extends Walker {
 	var $db_fields = array ('parent' => 'parent', 'id' => 'term_id');
 
 	/**
+	 * Start the element output.
+	 *
 	 * @see Walker::start_el()
 	 * @since 2.1.0
 	 *
-	 * @param string $output Passed by reference. Used to append additional content.
+	 * @param string $output   Passed by reference. Used to append additional content.
 	 * @param object $category Category data object.
-	 * @param int $depth Depth of category. Used for padding.
-	 * @param array $args Uses 'selected', 'show_count', and 'show_last_update' keys, if they exist.
+	 * @param int    $depth    Depth of category. Used for padding.
+	 * @param array  $args     Uses 'selected' and 'show_count' keys, if they exist. @see wp_dropdown_categories()
 	 */
-	function start_el(&$output, $category, $depth, $args) {
+	function start_el( &$output, $category, $depth = 0, $args = array(), $id = 0 ) {
 		$pad = str_repeat('&nbsp;', $depth * 3);
 
 		$cat_name = apply_filters('list_cats', $category->name, $category);
@@ -954,10 +966,6 @@ class Walker_CategoryDropdown extends Walker {
 		$output .= $pad.$cat_name;
 		if ( $args['show_count'] )
 			$output .= '&nbsp;&nbsp;('. $category->count .')';
-		if ( $args['show_last_update'] ) {
-			$format = 'Y-m-d';
-			$output .= '&nbsp;&nbsp;' . gmdate($format, $category->last_update_timestamp);
-		}
 		$output .= "</option>\n";
 	}
 }
@@ -994,7 +1002,7 @@ function get_tag_link( $tag ) {
  * @uses apply_filters() Calls 'get_the_tags' filter on the list of post tags.
  *
  * @param int $id Post ID.
- * @return array
+ * @return array|bool Array of tag objects on success, false on failure.
  */
 function get_the_tags( $id = 0 ) {
 	return apply_filters( 'get_the_tags', get_the_terms( $id, 'post_tag' ) );
@@ -1009,10 +1017,11 @@ function get_the_tags( $id = 0 ) {
  * @param string $before Optional. Before tags.
  * @param string $sep Optional. Between tags.
  * @param string $after Optional. After tags.
- * @return string
+ * @param int $id Optional. Post ID. Defaults to the current post.
+ * @return string|bool|WP_Error A list of tags on success, false or WP_Error on failure.
  */
-function get_the_tag_list( $before = '', $sep = '', $after = '' ) {
-	return apply_filters( 'the_tags', get_the_term_list( 0, 'post_tag', $before, $sep, $after ), $before, $sep, $after);
+function get_the_tag_list( $before = '', $sep = '', $after = '', $id = 0 ) {
+	return apply_filters( 'the_tags', get_the_term_list( $id, 'post_tag', $before, $sep, $after ), $before, $sep, $after, $id );
 }
 
 /**
@@ -1023,7 +1032,6 @@ function get_the_tag_list( $before = '', $sep = '', $after = '' ) {
  * @param string $before Optional. Before list.
  * @param string $sep Optional. Separate items using this.
  * @param string $after Optional. After list.
- * @return string
  */
 function the_tags( $before = null, $sep = ', ', $after = '' ) {
 	if ( null === $before )
@@ -1049,13 +1057,16 @@ function tag_description( $tag = 0 ) {
  * @since 2.8
  *
  * @param int $term Optional. Term ID. Will use global term ID by default.
+ * @param string $taxonomy Optional taxonomy name. Defaults to 'post_tag'.
  * @return string Term description, available.
  */
 function term_description( $term = 0, $taxonomy = 'post_tag' ) {
-	if ( !$term && ( is_tax() || is_tag() || is_category() ) ) {
+	if ( ! $term && ( is_tax() || is_tag() || is_category() ) ) {
 		$term = get_queried_object();
-		$taxonomy = $term->taxonomy;
-		$term = $term->term_id;
+		if ( $term ) {
+			$taxonomy = $term->taxonomy;
+			$term = $term->term_id;
+		}
 	}
 	$description = get_term_field( 'description', $term, $taxonomy );
 	return is_wp_error( $description ) ? '' : $description;
@@ -1064,33 +1075,23 @@ function term_description( $term = 0, $taxonomy = 'post_tag' ) {
 /**
  * Retrieve the terms of the taxonomy that are attached to the post.
  *
- * This function can only be used within the loop.
- *
  * @since 2.5.0
  *
- * @param int $id Post ID. Is not optional.
+ * @param int|object $post Post ID or object.
  * @param string $taxonomy Taxonomy name.
- * @return array|bool False on failure. Array of term objects on success.
+ * @return array|bool|WP_Error Array of term objects on success, false or WP_Error on failure.
  */
-function get_the_terms( $id = 0, $taxonomy ) {
-	global $post;
+function get_the_terms( $post, $taxonomy ) {
+	if ( ! $post = get_post( $post ) )
+		return false;
 
- 	$id = (int) $id;
-
-	if ( !$id ) {
-		if ( !$post->ID )
-			return false;
-		else
-			$id = (int) $post->ID;
-	}
-
-	$terms = get_object_term_cache( $id, $taxonomy );
+	$terms = get_object_term_cache( $post->ID, $taxonomy );
 	if ( false === $terms ) {
-		$terms = wp_get_object_terms( $id, $taxonomy );
-		wp_cache_add($id, $terms, $taxonomy . '_relationships');
+		$terms = wp_get_object_terms( $post->ID, $taxonomy );
+		wp_cache_add($post->ID, $terms, $taxonomy . '_relationships');
 	}
 
-	$terms = apply_filters( 'get_the_terms', $terms, $id, $taxonomy );
+	$terms = apply_filters( 'get_the_terms', $terms, $post->ID, $taxonomy );
 
 	if ( empty( $terms ) )
 		return false;
@@ -1108,9 +1109,9 @@ function get_the_terms( $id = 0, $taxonomy ) {
  * @param string $before Optional. Before list.
  * @param string $sep Optional. Separate items using this.
  * @param string $after Optional. After list.
- * @return string
+ * @return string|bool|WP_Error A list of terms on success, false or WP_Error on failure.
  */
-function get_the_term_list( $id = 0, $taxonomy, $before = '', $sep = '', $after = '' ) {
+function get_the_term_list( $id, $taxonomy, $before = '', $sep = '', $after = '' ) {
 	$terms = get_the_terms( $id, $taxonomy );
 
 	if ( is_wp_error( $terms ) )
@@ -1123,7 +1124,7 @@ function get_the_term_list( $id = 0, $taxonomy, $before = '', $sep = '', $after
 		$link = get_term_link( $term, $taxonomy );
 		if ( is_wp_error( $link ) )
 			return $link;
-		$term_links[] = '<a href="' . $link . '" rel="tag">' . $term->name . '</a>';
+		$term_links[] = '<a href="' . esc_url( $link ) . '" rel="tag">' . $term->name . '</a>';
 	}
 
 	$term_links = apply_filters( "term_links-$taxonomy", $term_links );
@@ -1143,7 +1144,7 @@ function get_the_term_list( $id = 0, $taxonomy, $before = '', $sep = '', $after
  * @param string $after Optional. After list.
  * @return null|bool False on WordPress error. Returns null when displaying.
  */
-function the_terms( $id = 0, $taxonomy, $before = '', $sep = ', ', $after = '' ) {
+function the_terms( $id, $taxonomy, $before = '', $sep = ', ', $after = '' ) {
 	$term_list = get_the_term_list( $id, $taxonomy, $before, $sep, $after );
 
 	if ( is_wp_error( $term_list ) )
@@ -1152,13 +1153,12 @@ function the_terms( $id = 0, $taxonomy, $before = '', $sep = ', ', $after = '' )
 	echo apply_filters('the_terms', $term_list, $taxonomy, $before, $sep, $after);
 }
 
-
 /**
  * Check if the current post has any of given category.
  *
  * @since 3.1.0
  *
- * @param string|int|array $tag Optional. The category name/term_id/slug or array of them to check for.
+ * @param string|int|array $category Optional. The category name/term_id/slug or array of them to check for.
  * @param int|object $post Optional. Post to check instead of the current post.
  * @return bool True if the current post has any of the given categories (or any category, if no category specified).
  */
@@ -1213,5 +1213,3 @@ function has_term( $term = '', $taxonomy = '', $post = null ) {
 
 	return $r;
 }
-
-?>
